import React from "react";
import ReactDOM from "react-dom";
import store from 'app/core/store';
import { config } from "app/core/config";
import { contextSrv } from 'app/core/services/context_srv';
import { dateTime } from '@grafana/data';
import { HasExpired, ExpiresSoon } from './LicenseWarning';
import { OrgRole } from 'app/types';

const SHOW_EXPIRE_WARNING_DAYS = 30;
const DISMISS_WARNING_FOR_DAYS = 5;
/* The auto dissmiss cannot be too low as it will also trigger the snooze */
const WARNING_CLOSE_TIMEOUT_SEC = 3600;
const LICENSE_WARNING_DISMISS_UNTIL_KEY = 'grafana.licence.warning.dismissUntil';

let warningContainer: HTMLElement | null = null;

export async function initLicenseWarnings() {
  const dismissUntil = store.get(LICENSE_WARNING_DISMISS_UNTIL_KEY);
  const hasDismissed = dismissUntil && dismissUntil > dateTime().valueOf();
  const isAdmin = contextSrv.hasRole(OrgRole.Admin);

  if (isExpired() || (isAdmin && willExpireSoon() && !hasDismissed)) {
    setTimeout(renderLicenseWarning, 1000);
  }
}

export function onCloseWarning() {
  const dismissTill = dateTime().add(DISMISS_WARNING_FOR_DAYS, 'd').valueOf();
  store.set(LICENSE_WARNING_DISMISS_UNTIL_KEY, dismissTill);
  getMainView().removeChild(warningContainer);
}

async function renderLicenseWarning() {
  warningContainer = document.createElement("div");

  if (isExpired()) {
    ReactDOM.render(<HasExpired />, warningContainer);
  } else if (willExpireSoon()) {
    const expiresIn = willExpireInDays();
    ReactDOM.render(<ExpiresSoon days={expiresIn} onCloseWarning={onCloseWarning} />, warningContainer);

    // out hide expire warning in case it' a TV monitor with admin permissions
    setTimeout(onCloseWarning, 1000 * WARNING_CLOSE_TIMEOUT_SEC);
  }

  getMainView().appendChild(warningContainer);
}

function getMainView(): Element {
  return document.getElementsByClassName("main-view")[0];
}

export function isExpired(): boolean {
  const { expiry } = config.licenseInfo;
  return !expiry || dateTime(expiry * 1000) < dateTime();
}

export function willExpireSoon(): boolean {
 const { expiry } = config.licenseInfo;
 return dateTime(expiry * 1000) < dateTime().add(SHOW_EXPIRE_WARNING_DAYS, 'd');
}

export function willExpireInDays(): number {
  const { expiry } = config.licenseInfo;
  return Math.ceil((expiry - dateTime().unix()) / 3600 / 24);
}
